import json

from django.test import TestCase
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser

from nose.tools import raises
from rest_framework.test import APIRequestFactory, force_authenticate

from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.projects.models import Project
from geokey.categories.tests.model_factories import (
    CategoryFactory, TextFieldFactory, NumericFieldFactory
)
from geokey.users.tests.model_factories import UserGroupF

from ..model_factories import (
    ObservationFactory, CommentFactory, LocationFactory
)

from geokey.contributions.views.observations import (
    SingleAllContributionAPIView, SingleContributionAPIView,
    ContributionSearchAPIView, ProjectObservations
)
from geokey.contributions.models import Observation


class ContributionSearchTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )
        category = CategoryFactory.create()
        TextFieldFactory.create(**{'key': 'key', 'category': category})

        ObservationFactory.create_batch(5, **{
            'properties': {'key': 'blah'},
            'project': self.project,
            'category': category
        })
        ObservationFactory.create_batch(5, **{
            'properties': {'key': 'blub'},
            'project': self.project,
            'category': category
        })

    def get(self, user, query):
        url = reverse('api:contributions_search', kwargs={
            'project_id': self.project.id
        })
        request = self.factory.get(url + '?query=' + query)
        force_authenticate(request, user=user)
        theview = ContributionSearchAPIView.as_view()
        return theview(request, project_id=self.project.id).render()

    def test_get_with_bl(self):
        response = self.get(self.admin, 'bl')
        self.assertEqual(response.status_code, 200)

        features = json.loads(response.content)
        self.assertEqual(len(features.get('features')), 10)

    def test_get_with_blah(self):
        response = self.get(self.admin, 'blah')
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, 'blub')

        features = json.loads(response.content)
        self.assertEqual(len(features.get('features')), 5)

    def test_get_with_blub(self):
        response = self.get(self.admin, 'blub')
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, 'blah')

        features = json.loads(response.content)
        self.assertEqual(len(features.get('features')), 5)


class SingleContributionAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.moderator = UserF.create()
        self.viewer = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator],
            add_viewer=[self.viewer]
        )
        self.moderators = UserGroupF(add_users=[self.moderator], **{
            'project': self.project,
            'can_moderate': True
        })
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.creator,
            'status': 'active'
        })

    def test_approve_pending_with_admin(self):
        self.observation.status = 'pending'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    def test_approve_pending_with_admin_empty_properties(self):
        self.observation.properties = None
        self.observation.status = 'pending'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    def test_suspend_pending_with_admin(self):
        self.observation.status = 'active'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "pending"}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    def test_approve_pending_with_moderator(self):
        self.observation.status = 'pending'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.moderator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    @raises(PermissionDenied)
    def test_approve_pending_with_contributor(self):
        self.observation.status = 'pending'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    def test_approve_pending_with_contributor_who_is_moderator(self):
        self.moderators.users.add(self.creator)
        self.observation.status = 'pending'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    def test_flag_with_admin(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "pending"}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    def test_flag_with_moderator(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "pending"}}
        request.user = self.moderator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'pending')

    def test_flag_with_moderator_and_edit(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {
            'properties': {
                'key': 'updated'
            },
            'meta': {
                'status': 'pending',
            }
        }
        request.user = self.moderator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'pending')
        self.assertEqual(ref.properties.get('key'), 'updated')

    def test_flag_with_contributor(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "pending"}}
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    @raises(PermissionDenied)
    def test_flag_with_anonymous(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "pending"}}
        request.user = AnonymousUser()

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    @raises(PermissionDenied)
    def test_update_user(self):
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'properies': {'text': 'blah'}}
        request.user = self.viewer

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)

    def test_update_under_review(self):
        CommentFactory.create(**{
            'commentto': self.observation,
            'review_status': 'open'
        })
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': 'active'}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)

        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'review')

    @raises(PermissionDenied)
    def test_commit_from_draft_admin(self):
        self.observation.status = 'draft'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.admin

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    @raises(PermissionDenied)
    def test_commit_from_draft_with_moderator(self):
        self.observation.status = 'draft'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.moderator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    def test_commit_from_draft_with_contributor(self):
        self.moderators.users.add(self.creator)

        self.observation.status = 'draft'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    def test_commit_from_draft_with_contributor_who_is_moderator(self):
        self.observation.status = 'draft'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {'meta': {'status': "active"}}
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'pending'
        )

    def test_commit_from_draft_with_contributor_with_data(self):
        self.observation.status = 'draft'
        self.observation.save()

        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.patch(url)
        request.DATA = {
            'properties': {
                'key': 'updated'
            },
            'meta': {
                'status': "active",
            }
        }
        request.user = self.creator

        view = SingleContributionAPIView()
        view.update_and_respond(request, self.observation)
        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'pending')
        self.assertEqual(ref.properties.get('key'), 'updated')


class SingleAllContributionAPIViewTest(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.creator,
            'status': 'active'
        })

    def test_get_object_with_creator(self):
        view = SingleAllContributionAPIView()
        view.get_object(
            self.creator, self.observation.project.id, self.observation.id)

    def test_get_object_with_admin(self):
        view = SingleAllContributionAPIView()
        observation = view.get_object(
            self.admin, self.observation.project.id, self.observation.id)
        self.assertEqual(observation, self.observation)

    @raises(Project.DoesNotExist)
    def test_get_object_with_some_dude(self):
        some_dude = UserF.create()
        view = SingleAllContributionAPIView()
        view.get_object(
            some_dude, self.observation.project.id, self.observation.id)

    @raises(Observation.DoesNotExist)
    def test_get_draft_object_with_admin(self):
        self.observation.status = 'draft'
        self.observation.save()

        view = SingleAllContributionAPIView()
        view.get_object(
            self.admin, self.observation.project.id, self.observation.id)

    def test_api_with_admin(self):
        CommentFactory.create_batch(5, **{'commentto': self.observation})
        factory = APIRequestFactory()
        url = reverse('api:project_single_observation', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = factory.get(url)
        force_authenticate(request, user=self.admin)
        theview = SingleAllContributionAPIView.as_view()
        response = theview(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id).render()
        self.assertEqual(response.status_code, 200)


class ProjectPublicApiTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

        self.category = CategoryFactory(**{
            'status': 'active',
            'project': self.project
        })

        TextFieldFactory.create(**{
            'key': 'key_1',
            'category': self.category,
            'required': True,
            'order': 1
        })
        NumericFieldFactory.create(**{
            'key': 'key_2',
            'category': self.category,
            'minval': 0,
            'maxval': 1000,
            'order': 2
        })

        self.data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            }
        }

    def _post(self, data, user):
        url = reverse(
            'api:project_observations',
            kwargs={
                'project_id': self.project.id
            }
        )
        request = self.factory.post(
            url, json.dumps(data), content_type='application/json')
        force_authenticate(request, user=user)
        view = ProjectObservations.as_view()
        return view(request, project_id=self.project.id).render()

    def test_contribute_with_wrong_category(self):
        self.data['meta']['category'] = 3864

        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_with_invalid(self):
        data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "properties": {
                "key_1": 12,
                "key_2": "jsdbdjhsb"
            },
            "meta": {
                "category": self.category.id,
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            }
        }

        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_with_invalid_number(self):
        data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "properties": {
                "key_1": 12,
                "key_2": 2000
            },
            "meta": {
                "category": self.category.id,
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            }
        }

        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_with_existing_location(self):
        location = LocationFactory()
        data = {
            "type": "Feature",
            "geometry": location.geometry.geojson,
            "location": {
                "id": location.id,
                "name": location.name,
                "description": location.description,
                "private": location.private
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
            }
        }

        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 201)

    def test_contribute_with_private_for_project_location(self):
        location = LocationFactory(**{
            'private': True,
            'private_for_project': self.project
        })

        data = {
            "type": "Feature",
            "geometry": location.geometry.geojson,
            "location": {
                "id": location.id,
                "name": location.name,
                "description": location.description,
                "private": location.private
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
            }
        }
        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 201)

    def test_contribute_with_wrong_project_location(self):
        project = ProjectF()
        location = LocationFactory(**{
            'private': True,
            'private_for_project': project
        })

        data = {
            "type": "Feature",
            "geometry": location.geometry.geojson,
            "location": {
                "id": location.id,
                "name": location.name,
                "description": location.description,
                "private": location.private
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
            }
        }

        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_with_private_location(self):
        location = LocationFactory(**{
            'private': True
        })

        data = {
            "type": "Feature",
            "geometry": location.geometry.geojson,
            "location": {
                "id": location.id,
                "name": location.name,
                "description": location.description,
                "private": location.private
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
            }
        }

        response = self._post(data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_valid_draft(self):
        self.data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
                "status": "draft"
            }
        }
        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 201)
        self.assertIn('"status":"draft"', response.content)

    def test_contribute_valid_draft_with_empty_required(self):
        self.data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "properties": {
                "key_1": None,
                "key_2": 12
            },
            "meta": {
                "category": self.category.id,
                "status": "draft"
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            }
        }
        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 201)
        self.assertIn('"status":"draft"', response.content)

    def test_contribute_invalid_draft(self):
        self.data = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.13404607772827148,
                    51.52439200896907
                ]
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 'Blah'
            },
            "meta": {
                "category": self.category.id,
                "status": "draft"
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True
            },
        }
        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 400)

    def test_contribute_to_public_everyone_with_Anonymous(self):
        self.project.everyone_contributes = 'true'
        self.project.isprivate = False
        self.project.save()

        response = self._post(self.data, AnonymousUser())
        self.assertEqual(response.status_code, 201)

    def test_contribute_to_public_with_admin(self):
        self.project.isprivate = False
        self.project.save()
        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 201)
        self.assertIn('"status":"active"', response.content)

    def test_contribute_to_public_with_contributor(self):
        self.project.isprivate = False
        self.project.save()

        response = self._post(self.data, self.contributor)
        self.assertEqual(response.status_code, 201)
        self.assertIn('"status":"pending"', response.content)

    def test_contribute_to_public_with_non_member(self):
        self.project.isprivate = False
        self.project.save()

        response = self._post(self.data, self.non_member)
        self.assertEqual(response.status_code, 403)

    def test_contribute_to_public_with_anonymous(self):
        self.project.isprivate = False
        self.project.save()

        response = self._post(self.data, AnonymousUser())
        self.assertEqual(response.status_code, 403)

    def test_contribute_to_private_with_admin(self):
        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(self.project.observations.all()), 1)

    def test_contribute_to_private_with_contributor(self):
        response = self._post(self.data, self.contributor)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(self.project.observations.all()), 1)

    def test_contribute_to_private_with_non_member(self):
        response = self._post(self.data, self.non_member)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_private_with_anonymous(self):
        response = self._post(self.data, AnonymousUser())
        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_inactive_with_admin(self):
        self.project.status = 'inactive'
        self.project.save()

        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_inactive_with_contributor(self):
        self.project.status = 'inactive'
        self.project.save()

        response = self._post(self.data, self.contributor)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_inactive_with_non_member(self):
        self.project.status = 'inactive'
        self.project.save()

        response = self._post(self.data, self.non_member)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_inactive_with_Anonymous(self):
        self.project.status = 'inactive'
        self.project.save()

        response = self._post(self.data, AnonymousUser())
        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(self.project.observations.all()), 0)

    def test_contribute_to_deleted_with_admin(self):
        self.project.status = 'deleted'
        self.project.save()

        response = self._post(self.data, self.admin)
        self.assertEqual(response.status_code, 404)

    def test_contribute_to_deleted_with_contributor(self):
        self.project.status = 'deleted'
        self.project.save()

        response = self._post(self.data, self.contributor)
        self.assertEqual(response.status_code, 404)

    def test_contribute_to_deleted_with_non_member(self):
        self.project.status = 'deleted'
        self.project.save()

        response = self._post(self.data, self.non_member)
        self.assertEqual(response.status_code, 404)

    def test_contribute_to_deleted_with_anonymous(self):
        self.project.status = 'deleted'
        self.project.save()

        response = self._post(self.data, AnonymousUser())
        self.assertEqual(response.status_code, 404)


class GetSingleObservationInProject(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.observation = ObservationFactory(
            **{'project': self.project, 'creator': self.contributor})

    def _get(self, user):
        url = reverse(
            'api:project_single_observation',
            kwargs={
                'project_id': self.project.id,
                'observation_id': self.observation.id
            }
        )
        request = self.factory.get(url)
        force_authenticate(request, user=user)
        view = SingleAllContributionAPIView.as_view()
        return view(
            request, project_id=self.project.id,
            observation_id=self.observation.id).render()

    def test_get_with_admin(self):
        response = self._get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        response = self._get(self.contributor)
        self.assertEqual(response.status_code, 200)

    def test_get_with_non_member(self):
        user = UserF.create()
        response = self._get(user)
        self.assertEqual(response.status_code, 404)


class UpdateObservationInProject(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.category = CategoryFactory(**{
            'status': 'active',
            'project': self.project
        })

        TextFieldFactory.create(**{
            'key': 'key_1',
            'category': self.category,
            'order': 0
        })
        NumericFieldFactory.create(**{
            'key': 'key_2',
            'category': self.category,
            'order': 1
        })

        location = LocationFactory()

        self.observation = ObservationFactory.create(**{
            'properties': {
                "key_1": "value 1",
                "key_2": 12,
            },
            'category': self.category,
            'project': self.project,
            'location': location,
            'creator': self.admin,
            'status': 'active'
        })

        self.update_data = {
            "properties": {
                "version": 1,
                "key_2": 15
            }
        }

    def _patch(self, data, user):
        url = reverse(
            'api:project_single_observation',
            kwargs={
                'project_id': self.project.id,
                'observation_id': self.observation.id
            }
        )
        request = self.factory.patch(
            url, json.dumps(data), content_type='application/json')
        force_authenticate(request, user=user)
        view = SingleAllContributionAPIView.as_view()
        return view(
            request, project_id=self.project.id,
            observation_id=self.observation.id).render()

    def _delete(self, user):
        url = reverse(
            'api:project_single_observation',
            kwargs={
                'project_id': self.project.id,
                'observation_id': self.observation.id
            }
        )
        request = self.factory.delete(url, content_type='application/json')
        force_authenticate(request, user=user)
        view = SingleAllContributionAPIView.as_view()
        return view(
            request, project_id=self.project.id,
            observation_id=self.observation.id).render()

    def test_update_conflict(self):
        response = self._patch(
            self.update_data,
            self.admin
        )
        self.assertEqual(response.status_code, 200)

        data = {"properties": {"attributes": {"version": 1, "key_2": 2}}}
        response = self._patch(
            data,
            self.admin
        )
        self.assertEqual(response.status_code, 200)

    def test_update_location_with_admin(self):
        self.update_data['geometry'] = {
            'type': 'Point',
            'coordinates': [
                -0.1444154977798462,
                51.54671869005856
            ]
        }
        self.update_data['properties']['location'] = {
            'name': 'New name'
        }
        response = self._patch(
            self.update_data,
            self.admin
        )
        self.assertEqual(response.status_code, 200)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(
            observation.properties.get('key_2'), 15)

        self.assertContains(response, 'New name')
        self.assertContains(response, '-0.144415')

    def test_update_with_admin(self):
        response = self._patch(
            self.update_data,
            self.admin
        )
        self.assertEqual(response.status_code, 200)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(
            observation.properties.get('key_2'), 15)

    @raises(Observation.DoesNotExist)
    def test_delete_with_admin(self):
        response = self._delete(
            self.admin
        )

        self.assertEqual(response.status_code, 204)
        Observation.objects.get(pk=self.observation.id)

    def test_update_with_contributor(self):
        response = self._patch(
            self.update_data,
            self.contributor
        )
        self.assertEqual(response.status_code, 403)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(
            observation.properties.get('key_2'), 12)

    def test_delete_with_contributor(self):
        response = self._delete(
            self.contributor
        )
        self.assertEqual(response.status_code, 403)

    def test_update_with_non_member(self):
        response = self._patch(
            self.update_data,
            self.non_member
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.observation.properties.get('key_2'), 12)

    def test_delete_with_non_member(self):
        response = self._delete(
            self.non_member
        )
        self.assertEqual(response.status_code, 404)
        self.assertNotEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'deleted'
        )


class TestProjectPublicApi(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

    def get(self, user):
        url = reverse('api:project_observations', kwargs={
            'project_id': self.project.id
        })
        request = self.factory.get(url)
        force_authenticate(request, user=user)
        theview = ProjectObservations.as_view()
        return theview(
            request,
            project_id=self.project.id).render()

    def test_get_with_admin(self):
        response = self.get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        response = self.get(self.contributor)
        self.assertEqual(response.status_code, 200)

    def test_get_with_some_dude(self):
        some_dude = UserF.create()
        response = self.get(some_dude)
        self.assertEqual(response.status_code, 404)

    def test_get_with_anonymous(self):
        response = self.get(AnonymousUser())
        self.assertEqual(response.status_code, 404)



from django.contrib.auth.models import BaseUserManager
from django.utils import timezone


class UserManager(BaseUserManager):
    """
    Custom manger for geokey.user.models.User
    """
    def create_user(self, email, display_name, password=None, is_active=True,
                    **extra_fields):
        """
        Creates a new user in the data base

        Parameter
        ---------
        email : str
            Email address of the new user
        display_name : str
            display name of the new user
        password : str
            password of the new user
        is_active : Boolean
            indicates if the user is active

        Returns
        -------
        geokey.user.models.User
            The user that has been created
        """
        now = timezone.now()
        email = UserManager.normalize_email(email)

        user = self.model(email=email, display_name=display_name,
                          is_active=is_active, last_login=now, date_joined=now,
                          **extra_fields)

        user.set_password(password)
        user.save(using=self._db)

        return user

    def create_superuser(self, email, display_name, password, **extra_fields):
        """
        Creates a new superuser

        Parameter
        ---------
        email : str
            Email address of the new user
        display_name : str
            display name of the new user
        password : str
            password of the new user

        Returns
        -------
        geokey.user.models.User
            The user that has been created
        """
        user = self.create_user(email, display_name, password=password,
                                **extra_fields)
        user.is_superuser = True
        user.save()
        return user

    def get_by_natural_key(self, username):
        """
        Returns the user identified by email. Overwrites parent method to
        make user names not case-sensitive.

        Parameter
        ---------
        username : str
            Email address to identify the user

        Returns
        -------
        geokey.user.models.User
        """
        return self.get(email__iexact=username)



from django.db.models import Q

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from geokey.core.decorators import handle_exceptions_for_ajax

from ..models import Location
from ..serializers import LocationSerializer


class Locations(APIView):
    """
    Public API endpoint for all locations in a project.
    /api/projects/:project_id/locations/
    """

    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        Returns a list of locations that can be used for contributions to the
        given project.

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project the locations should be returned for

        Returns
        -------
        rest_framework.response.Respone
            Contains the list of serialised locations
        """
        q = request.GET.get('query')
        locations = Location.objects.get_list(request.user, project_id)

        if q is not None:
            locations = locations.filter(
                Q(name__icontains=q.lower()) |
                Q(description__icontains=q.lower())
            )

        serializer = LocationSerializer(locations, many=True)
        return Response(serializer.data)


class SingleLocation(APIView):
    """
    Public API endpoint for a single location in a project.
    /api/projects/:project_id/locations/:location_id/
    """
    @handle_exceptions_for_ajax
    def patch(self, request, project_id, location_id):
        """
        Updates a single location

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project in the database
        location_id : int
            identifies the location in the database

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised location
        """
        location = Location.objects.get_single(
            request.user, project_id, location_id)
        serializer = LocationSerializer(
            location, data=request.DATA, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



from django.test import TestCase

from geokey.contributions.templatetags import kml_tags


class TemplateTagsTest(TestCase):
    def test_kml_name(self):
        name = kml_tags.kml_name({'display_field': {'value': 'Awesome pub'}})
        self.assertEqual(name, 'Awesome pub')

    def test_geom(self):
        data = {
            "location": {
                "geometry": {"type": "Point", "coordinates": [-2.812, 51.179]}
            }
        }
        kml_geom = kml_tags.kml_geom(data)
        self.assertEqual(
            kml_geom,
            '<Point><coordinates>-2.812,51.179</coordinates></Point>'
        )

    def test_description(self):
        data = {
            'properties': {
                'key_1': 1,
                'key_2': 'value'
            }
        }
        description = kml_tags.kml_desc(data)
        self.assertEqual(
            description,
            '<![CDATA[<table><tr><td>key_1</td><td>1</td></tr>'
            '<tr><td>key_2</td><td>value</td></tr></table>]]>'
        )

    def test_style(self):
        data = {
            'meta': {
                'category': {
                    'colour': '#ff0000'
                }
            }
        }
        colour = kml_tags.kml_style(data)
        self.assertEqual(colour, 'ff0000')



from django.conf import settings
from django.conf.urls import patterns, include, url
from django.views.generic.base import RedirectView


urlpatterns = patterns(
    '',
    url(r'^ajax/', include('geokey.core.url.ajax', namespace="ajax")),
    url(r'^admin/', include('geokey.core.url.admin', namespace="admin")),
    url(r'^api/', include('geokey.core.url.api', namespace="api")),
    url(r'^oauth2/', include('oauth2_provider.urls', namespace='oauth2_provider')),
    url(r'^admin/account/', include('allauth.urls')),
    # url(r'^', include('core.url.extensions')),
    url(r'^$', RedirectView.as_view(url='/admin/', permanent=True)),
)

urlpatterns += patterns(
    '',
    (r'^media/(?P<path>.*)$',
        'django.views.static.serve',
        {'document_root': settings.MEDIA_ROOT}),
)



import json
from django.test import TestCase
from django.template.loader import render_to_string

from geokey.contributions.renderer.geojson import GeoJsonRenderer
from geokey.contributions.renderer.kml import KmlRenderer


class KmlRendererTest(TestCase):
    def setUp(self):
        self.contrib = {
            'id': 1,
            'location': {
                'id': 1,
                'name': 'Location',
                'description': None,
                'geometry': '{"type": "Point","coordinates": [ '
                            '-0.144415497779846, 51.54671869005856]}'
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "display_field": {
                "key": "name",
                "value": "The Grafton"
            },
            "meta": {
                "creator": {
                    "id": 2,
                    "display_name": "Oliver"
                },
                "isowner": True,
                "updator": None,
                "status": "active",
                "created_at": "2014-09-19T15:51:32.804Z",
                "updated_at": "2014-09-21T15:51:32.804Z",
                "version": 1,
                "category": {
                    "id": 40,
                    "name": "Pubs",
                    "description": "",
                    "status": "active",
                    "fields": [
                        {
                            "id": 117,
                            "name": "Name",
                            "key": "name",
                            "fieldtype": "TextField",
                            "description": "",
                            "status": "active",
                            "required": True
                        },
                        {
                            "id": 118,
                            "name": "Address",
                            "key": "address",
                            "fieldtype": "TextField",
                            "description": "",
                            "status": "active",
                            "required": False
                        },
                        {
                            "id": 119,
                            "name": "Child friedly",
                            "key": "child_friedly",
                            "fieldtype": "TrueFalseField",
                            "description": "Would your take your kids?",
                            "status": "active",
                            "required": False
                        }
                    ],
                    "colour": "#0033ff",
                    "created_at": "2014-09-17T00:00:00Z"
                }
            },
            "comments": [],
            "review_comments": [],
            "media": []
        }

    def test_render(self):
        renderer = KmlRenderer()
        result = renderer.render([self.contrib])

        rendered = render_to_string(
            'geometries/placemarks.kml',
            {'data': [self.contrib]}
        )
        self.assertEqual(result, rendered)


class GeoJsonRendererTest(TestCase):
    def setUp(self):
        self.contrib = {
            'id': 1,
            'location': {
                'id': 1,
                'name': 'Location',
                'description': None,
                'geometry': '{"type": "Point","coordinates": [ '
                            '-0.144415497779846, 51.54671869005856]}'
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "creator": {
                    "id": 2,
                    "display_name": "Oliver"
                },
                "isowner": True,
                "updator": None,
                "status": "active",
                "created_at": "2014-09-19T15:51:32.804Z",
                "updated_at": "2014-09-21T15:51:32.804Z",
                "version": 1,
                "category": {
                    "id": 40,
                    "name": "Pubs",
                    "description": "",
                    "status": "active",
                    "fields": [
                        {
                            "id": 117,
                            "name": "Name",
                            "key": "name",
                            "fieldtype": "TextField",
                            "description": "",
                            "status": "active",
                            "required": True
                        },
                        {
                            "id": 118,
                            "name": "Address",
                            "key": "address",
                            "fieldtype": "TextField",
                            "description": "",
                            "status": "active",
                            "required": False
                        },
                        {
                            "id": 119,
                            "name": "Child friedly",
                            "key": "child_friedly",
                            "fieldtype": "TrueFalseField",
                            "description": "Would your take your kids?",
                            "status": "active",
                            "required": False
                        }
                    ],
                    "colour": "#0033ff",
                    "created_at": "2014-09-17T00:00:00Z"
                }
            },
            "comments": [],
            "review_comments": [],
            "media": []
        }

    def test_render_single(self):
        renderer = GeoJsonRenderer()
        result = renderer.render_single(self.contrib)

        self.assertTrue('geometry' in result)
        self.assertFalse('geometry' in result.get('location'))

    def test_render_many(self):
        renderer = GeoJsonRenderer()
        result = renderer.render_many([self.contrib])

        self.assertEqual(result.get('type'), 'FeatureCollection')
        self.assertEqual(len(result.get('features')), 1)

    def test_render_with_none(self):
        renderer = GeoJsonRenderer()
        result = renderer.render(None)

        self.assertEqual(result, '')

    def test_render_with_single(self):
        renderer = GeoJsonRenderer()
        result = json.loads(renderer.render(self.contrib))

        self.assertTrue('geometry' in result)
        self.assertFalse('geometry' in result.get('location'))

    def test_render_with_many(self):
        renderer = GeoJsonRenderer()
        result = json.loads(renderer.render([self.contrib]))

        self.assertEqual(result.get('type'), 'FeatureCollection')
        self.assertEqual(len(result.get('features')), 1)



"""
Django settings for opencomap project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
from os.path import abspath, dirname, join, normpath
from django.contrib import messages

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DJANGO_ROOT = dirname(dirname(abspath(__file__)))
SITE_ROOT = dirname(DJANGO_ROOT)
TEMPLATE_DIRS = (
    normpath(join(SITE_ROOT, 'templates')),
)
STATICFILES_DIRS = (
    normpath(join(SITE_ROOT, 'static')),
)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# Application definition

INSTALLED_APPS = (
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.gis',
    'django.contrib.sites',

    # third-party apps
    'django_hstore',
    'oauth2_provider',
    'easy_thumbnails',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',

    # geokey apps
    'geokey.projects',
    'geokey.categories',
    'geokey.contributions',
    'geokey.users',
    'geokey.applications',
    'geokey.superusertools',
    'geokey.extensions',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'geokey.core.middleware.XsSharing',
    # 'core.middleware.TerminalLogging',
)

OAUTH2_PROVIDER = {
    # this is the list of available scopes
    'SCOPES': {'read': 'Read scope', 'write': 'Write scope'},
    'CLIENT_SECRET_GENERATOR_LENGTH': 40
}
OAUTH2_PROVIDER_APPLICATION_MODEL = 'applications.Application'

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        # 'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'oauth2_provider.ext.rest_framework.OAuth2Authentication',
    ),
}

MESSAGE_TAGS = {
    messages.DEBUG: 'debug',
    messages.INFO: 'info',
    messages.SUCCESS: 'success',
    messages.WARNING: 'warning',
    messages.ERROR: 'danger'
}

AUTH_USER_MODEL = 'users.User'
AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend"
)

SITE_ID = 1
LOGIN_REDIRECT_URL = '/admin/dashboard/'
LOGIN_URL = '/admin/account/login/'
ACCOUNT_LOGOUT_REDIRECT_URL = '/admin/account/login/'
ACCOUNT_USER_MODEL_USERNAME_FIELD = 'display_name'
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = True
ACCOUNT_AUTHENTICATION_METHOD = 'email'
ACCOUNT_LOGOUT_ON_GET = True
ACCOUNT_CONFIRM_EMAIL_ON_GET = True
ACCOUNT_EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL = '/admin/account/login/'
ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL = '/admin/dashboard/'
ACCOUNT_FORMS = {
    'signup': 'geokey.users.forms.UserRegistrationForm',
    'change_password': 'geokey.users.forms.CustomPasswordChangeForm',
    'reset_password_from_key': 'geokey.users.forms.CustomResetPasswordKeyForm'
}

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.request",
    "django.contrib.auth.context_processors.auth",
    "geokey.core.context_processors.project_settings",
    "django.contrib.messages.context_processors.messages",
    "allauth.account.context_processors.account",
    "allauth.socialaccount.context_processors.socialaccount",
)

APPEND_SLASH = True

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

ENABLE_VIDEO = False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0002_auto_20150106_1338'),
    ]

    operations = [
        migrations.CreateModel(
            name='DateField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
            ],
            options={
            },
            bases=('categories.field',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def clean_youtube_links(apps, schema_editor):
    VideoFile = apps.get_model("contributions", "VideoFile")

    for file in VideoFile.objects.all():
        new_link = file.youtube_link.replace('http://', 'https://')
        file.youtube_link = new_link
        file.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0010_auto_20150511_1132'),
    ]

    operations = [
        migrations.RunPython(clean_youtube_links),
    ]



import datetime
import factory

from django.contrib.gis.geos import GEOSGeometry

from geokey.users.tests.model_factories import UserF

from geokey.projects.models import Project, Admins


class ProjectF(factory.django.DjangoModelFactory):
    class Meta:
        model = Project

    name = factory.Sequence(lambda n: 'project %d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    isprivate = True
    everyone_contributes = 'false'
    created_at = datetime.date(2014, 11, 11)
    creator = factory.SubFactory(UserF)
    status = 'active'
    geographic_extend = GEOSGeometry(
        '{"type": "Polygon","coordinates": [[[-0.508,51.682],[-0.53,51.327],'
        '[0.225,51.323],[0.167,51.667],[-0.508,51.682]]]}'
    )

    @factory.post_generation
    def add_admins(self, create, extracted, **kwargs):
        if not create:
            return

        Admins.objects.create(project=self, user=self.creator)
        if extracted:
            for user in extracted:
                Admins.objects.create(project=self, user=user)

    @factory.post_generation
    def add_moderators(self, create, extracted, **kwargs):
        from geokey.users.tests.model_factories import UserGroupF
        if not create:
            return

        if extracted:
            UserGroupF(add_users=extracted, **{
                'project': self,
                'can_moderate': True
            })

    @factory.post_generation
    def add_contributors(self, create, extracted, **kwargs):
        from geokey.users.tests.model_factories import UserGroupF
        if not create:
            return

        if extracted:
            UserGroupF(add_users=extracted, **{
                'project': self,
                'can_contribute': True
            })

    @factory.post_generation
    def add_viewer(self, create, extracted, **kwargs):
        from geokey.users.tests.model_factories import UserGroupF
        if not create:
            return

        if extracted:
            UserGroupF(add_users=extracted, **{
                'project': self,
                'can_contribute': False,
                'can_moderate': False,
            })


class AdminsFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Admins

    project = factory.SubFactory(ProjectF)
    user = factory.SubFactory(UserF)
    contact = True



import json

import time
from iso8601 import parse_date
from iso8601.iso8601 import ParseError

from django.conf import settings
from django.db import models
from django.db.models.loading import get_model

from geokey.core.exceptions import InputError

from .manager import CategoryManager, FieldManager, LookupValueManager
from .base import STATUS, DEFAULT_STATUS


class Category(models.Model):
    """
    Defines the data structure of a certain type of features.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    project = models.ForeignKey('projects.Project', related_name='categories')
    created_at = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)
    order = models.IntegerField(default=0)
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )
    display_field = models.ForeignKey(
        'categories.Field',
        null=True,
        related_name='display_field_of'
    )
    default_status = models.CharField(
        choices=DEFAULT_STATUS,
        default=DEFAULT_STATUS.pending,
        max_length=20
    )
    colour = models.TextField(default='#0033ff')
    symbol = models.ImageField(upload_to='symbols', null=True)

    objects = CategoryManager()

    class Meta:
        ordering = ['order']

    def re_order_fields(self, order):
        """
        Changes the order in which fields are displayed on client side.

        Parameters
        -------
        order : List
            IDs of fields, ordered according to new display order
        """
        fields_to_save = []
        for idx, field_id in enumerate(order):
            field = self.fields.get(pk=field_id)
            field.order = idx
            fields_to_save.append(field)

        for field in fields_to_save:
            field.save()

    def get_query(self, rule):
        """
        Returns the SQL where clause for the category. It combines the where
        clause parts of each field in the category.

        Returns
        -------
        str
            SQL where clause for the rule
        """
        queries = ['(category_id = %s)' % self.id]

        if 'min_date' in rule:
            queries.append('(created_at >= to_date(\'' +
                           rule['min_date'] +
                           '\', \'YYYY-MM-DD HH24:MI\'))')

        if 'max_date' in rule:
            queries.append('(created_at <= to_date(\'' +
                           rule['max_date'] +
                           '\', \'YYYY-MM-DD HH24:MI\'))')

        for key in rule:
            if key not in ['min_date', 'max_date']:
                field = self.fields.get_subclass(key=key)
                queries.append(field.get_filter(rule[key]))

        return '(%s)' % ' AND '.join(queries)

    def delete(self):
        """
        Deletes the category by setting its status to deleted.

        Notes
        -----
        It also deletes all contributions of that category.
        """
        from geokey.contributions.models import Observation
        Observation.objects.filter(category=self).delete()

        groups = self.project.usergroups.all()
        for usergroup in groups:
            if usergroup.filters is not None:
                f = usergroup.filters.pop(str(self.id), None)
                if f is not None:
                    usergroup.save()

        self.status = STATUS.deleted
        self.save()


class Field(models.Model):
    """
    A Field defines data type of one characterictic of an obesrvation. Used to
    create forms of user interfaces and to validate user inputs.
    """
    name = models.CharField(max_length=100)
    key = models.CharField(max_length=103)
    description = models.TextField(null=True, blank=True)
    required = models.BooleanField(default=False)
    category = models.ForeignKey('Category', related_name='fields')
    order = models.IntegerField(default=0)
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )

    objects = FieldManager()

    class Meta:
        unique_together = ('key', 'category')

    @classmethod
    def create(self, name, key, description, required, category, field_type):
        """
        Creates a new field based on the field type provided.

        Parameters
        ----------
        name : str
            Name of the field. Used for displaying labels.
        key : str
            Key of the field. Used in API to assign values to a field.
        description : str
            Long description providing further details about the field.
        required : Boolean
            Indicates if the field is required
        category : geokey.categories.models.Category
            The category this field is assigned to.
        field_type : str
            Type of the field. Must be one of:
                - TextField
                - NumericField
                - DateField
                - DateTimeField
                - TimeField
                - LookupField
                - MultipleLookupfield

        Returns
        -------
        geokey.categories.models.Field
            Intance of the newly created field
        """
        model_class = get_model('categories', field_type)
        order = category.fields.count()

        field = model_class.objects.create(
            name=name,
            key=key,
            description=description,
            required=required,
            category=category,
            order=order
        )
        field.save()

        if order == 0:
            category.display_field = field
            category.save()

        return field

    @classmethod
    def get_field_types(cls):
        """
        Returns the names of the subclasses of Field.

        Returns
        -------
        List
            The names of available field types.
        """
        return cls.__subclasses__()

    def validate_input(self, value):
        """
        Validates the given value against the field definition.
        @abstractmethod
        """
        raise NotImplementedError(
            'The method `validate_input` has not been implemented for this '
            'child class of Field.'
        )

    def validate_required(self, value):
        """
        Validates input value against required status.
        Raises an `InputError` if no value has been provided.
        """
        if self.status == STATUS.active and self.required and (value is None):
            raise InputError('The field %s is required.' % self.name)

    def convert_from_string(self, value):
        """
        Converts the given value of an Observation's field from String
        to the proper data type. By default returns simply the value in
        String format. Needs to be overridden in order to support other data
        types.

        Notes
        -----
        Deprecated from version 0.6 on.
        """
        return value

    @property
    def fieldtype(self):
        """
        Returns the class name of the field instance

        Returns
        -------
        str
            The class name of the field instance
        """
        return self.__class__.__name__

    @property
    def type_name(self):
        """
        Returns the type name of the field instance. This is a human-readable
        name that can be used in user interfaces, e.g. Date and Time for
        DateTimeField.
        @abstractmethod
        """
        raise NotImplementedError(
            'The property `type_name` has not been implemented for this '
            'subclass of Field.'
        )

    def get_filter(self, rule):
        """
        Returns an SQL where clause that can be used to filter contributions in
        data groupings
        @abstractmethod

        Parameter
        ---------
        rule : str or list or dict
            Depending on the field type, this provides the values the filter
            should be built against

        Return
        ------
        str
            The where-clause that can be used in SQL queries.
        """
        raise NotImplementedError(
            'The method `filter` has not been implemented for this '
            'subclass of Field.'
        )

    def delete(self):
        """
        Deletes the field.

        Notes
        -----
        Also deletes all references to the Field in Rules.
        """

        groups = self.category.project.usergroups.all()
        for usergroup in groups:
            if usergroup.filters is not None:
                category_filter = usergroup.filters.get(
                    str(self.category.id), None)

                if category_filter is not None:
                    field_filter = category_filter.pop(self.key, None)

                    if field_filter is not None:
                        usergroup.save()

        super(Field, self).delete()


class TextField(Field):
    """
    A field for character strings.
    """
    textarea = models.BooleanField(default=False)
    maxlength = models.IntegerField(blank=True, null=True)

    def validate_required(self, value):
        """
        Validate the given value agaist required status. Checks if value is
        not None and has at least one character.

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if no value is provided
        """
        if isinstance(value, str) or isinstance(value, unicode):
            value = value.encode('utf-8')

        if self.status == STATUS.active and self.required and (
                value is None or len(str(value)) == 0):
            raise InputError('The field %s is required.' % self.name)

    def validate_input(self, value):
        """
        Validates if the given value is a valid input for the TextField.
        Checks if the value is required and if maxlength constraint is met.

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(value)

        if value is not None:
            if self.maxlength is not None and len(value) > self.maxlength:
                raise InputError('The input provided for text field %s '
                                 'contains too many characters.' % self.name)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Text'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : str
            A keyword that needs to matched for the filter to apply.

        Return
        ------
        str
            SQL where-clause
        """
        return ('((properties ->> \'' + self.key + '\') '
                'ILIKE \'%%' + rule + '%%\')')


class NumericField(Field):
    """
    A field for numeric values.
    """
    minval = models.FloatField(blank=True, null=True)
    maxval = models.FloatField(blank=True, null=True)

    def validate_input(self, value):
        """
        Validates if the given value is a valid input for the NumerField.
        Checks if a value of type number has been provided or if a value of
        type String has been provided that can be successfully converted to a
        Float value. Then checks if the value is between bounds of minval and
        maxval.

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        if isinstance(value, (str, unicode)) and len(value) == 0:
            value = None

        self.validate_required(value)

        if value is not None:
            if isinstance(value, (str, unicode)):
                try:
                    value = float(value) if '.' in value else int(value)
                except ValueError:
                    raise InputError(
                        'The value provided for field %s is not a number.' %
                        self.name
                    )

            if isinstance(value, (int, long, float, complex)):
                if self.minval and self.maxval and (
                        not (value >= self.minval) and (value <= self.maxval)):
                    raise InputError('The value provided for field %s must be '
                                     ' greater than %s and lower than %s.'
                                     % (self.name, self.minval, self.maxval))

                else:
                    if self.minval and (not (value >= self.minval)):
                        raise InputError('The value provided for field %s must'
                                         ' be greater than %s.'
                                         % (self.name, self.minval))

                    if self.maxval and (not (value <= self.maxval)):
                        raise InputError('The value provided for field %s must'
                                         ' be lower than %s.'
                                         % (self.name, self.maxval))

    def convert_from_string(self, value):
        """
        Returns the `value` of the field either as float or int.

        Parameters
        ----------
        value : str
            The value to be converted

        Return
        ------
        int or float
            Value of the field

        Notes
        -----
        Deprecated from version 0.6 on.
        """
        if value is None or len(value) == 0:
            return None

        try:
            return int(value)
        except ValueError:
            return float(value)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Numeric'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : dict
            Contains either minimum and maximum value for the filer:
            {minval: 1, maxval: 10}

        Return
        ------
        str
            SQL where-clause
        """
        minval = rule.get('minval')
        maxval = rule.get('maxval')

        if minval is not None and maxval is not None:
            return ('(cast(properties ->> \'%s\' as double precision) >= %s) '
                    'AND (cast(properties ->> \'%s\' as double precision) <= '
                    '%s)' % (self.key, minval, self.key, maxval))
        else:
            if minval is not None:
                return ('(cast(properties ->> \'%s\' as double '
                        'precision) >= %s)' % (self.key, minval))

            if maxval is not None:
                return ('(cast(properties ->> \'%s\' as double '
                        'precision) <= %s)' % (self.key,  maxval))


class DateTimeField(Field):
    """
    A field for storing dates and times.
    """

    def validate_required(self, value):
        """
        Validate the given value agaist required status. Checks if value is
        not None and has at least one character.

        Parameters
        ----------
        value : str or None
            The value to be validated

        Notes
        -----
        Raises InputError if no value is provided
        """
        if (self.status == STATUS.active and
                self.required and (value is None or len(value) == 0)):
            raise InputError('The field %s is required.' % self.name)

    def validate_input(self, value):
        """
        Checks if the provided value is a valid and ISO8601 compliant date
        string.

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(value)
        if value is not None:
            try:
                parse_date(value)
            except ParseError:
                raise InputError('The value for DateField %s is not a '
                                 'valid date.' % self.name)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Date and Time'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : dict
            Contains either minimum and maximum value for the filer:
            {minval: '2015-10-01', maxval: '2015-10-31'}

        Return
        ------
        str
            SQL where-clause
        """
        minval = rule.get('minval')
        maxval = rule.get('maxval')

        if minval is not None and maxval is not None:
            return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD HH24:MI\') '
                    '>= to_date(\'%s\', \'YYYY-MM-DD HH24:MI\')) AND '
                    '(to_date(properties ->> \'%s\', \'YYYY-MM-DD HH24:MI\') '
                    '<= to_date(\'%s\', \'YYYY-MM-DD HH24:MI\'))' %
                    (self.key, minval, self.key, maxval))
        else:
            if minval is not None:
                return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD '
                        'HH24:MI\') >= to_date(\'%s\', \'YYYY-MM-DD HH24:MI\''
                        '))' % (self.key, minval))

            if maxval is not None:
                return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD '
                        'HH24:MI\') <= to_date(\'%s\', \'YYYY-MM-DD HH24:MI\''
                        '))' % (self.key, maxval))


class DateField(Field):
    """
    A field for storing dates.
    """
    def validate_required(self, value):
        """
        Validate the given value agaist required status. Checks if value is
        not None and has at least one character.

        Parameters
        ----------
        value : str or None
            The value to be validated

        Notes
        -----
        Raises InputError if no value is provided
        """
        if (self.status == STATUS.active and self.required and
                (value is None or len(value) == 0)):
            raise InputError('The field %s is required.' % self.name)

    def validate_input(self, value):
        """
        Checks if the provided value is a valid and ISO8601 compliant date
        string.

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(value)
        if value is not None:
            try:
                parse_date(value)
            except ParseError:
                raise InputError('The value for DateField %s is not a '
                                 'valid date.' % self.name)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Date'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : dict
            Contains either minimum and maximum value for the filer:
            {minval: '2015-10-01 10:00', maxval: '2015-10-31 15:00'}

        Return
        ------
        str
            SQL where-clause
        """
        minval = rule.get('minval')
        maxval = rule.get('maxval')

        if minval is not None and maxval is not None:
            return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD\') '
                    '>= to_date(\'%s\', \'YYYY-MM-DD\')) AND '
                    '(to_date(properties ->> \'%s\', \'YYYY-MM-DD\') '
                    '<= to_date(\'%s\', \'YYYY-MM-DD\'))' %
                    (self.key, minval, self.key, maxval))
        else:
            if minval is not None:
                return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD'
                        '\') >= to_date(\'%s\', \'YYYY-MM-DD\''
                        '))' % (self.key, minval))

            if maxval is not None:
                return ('(to_date(properties ->> \'%s\', \'YYYY-MM-DD'
                        '\') <= to_date(\'%s\', \'YYYY-MM-DD\''
                        '))' % (self.key, maxval))


class TimeField(Field):
    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Time'

    def validate_required(self, value):
        """
        Validate the given value agaist required status. Checks if value is
        not None and has at least one character.

        Parameters
        ----------
        value : str or None
            The value to be validated

        Notes
        -----
        Raises InputError if no value is provided
        """
        if (self.status == STATUS.active and self.required and
                (value is None or len(value) == 0)):
            raise InputError('The field %s is required.' % self.name)

    def validate_input(self, value):
        """
        Checks if the provided value is a matches the patterns HH:mm

        Parameters
        ----------
        value : str
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(value)
        if value is not None:
            try:
                time.strptime(value, '%H:%M')
            except ValueError:
                raise InputError('The value for TimeField %s is not a '
                                 'valid time.' % self.name)

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : dict
            Contains either minimum and maximum value for the filer:
            {minval: '10:00', maxval: '15:00'}

        Return
        ------
        str
            SQL where-clause
        """
        minval = rule.get('minval')
        maxval = rule.get('maxval')

        if minval is not None and maxval is not None:
            if time.strptime(minval, '%H:%M') > time.strptime(maxval, '%H:%M'):
                return ('((properties ->> \'%s\')::time >= \'%s\'::time) OR '
                        '((properties ->> \'%s\')::time <= \'%s\'::time)' %
                        (self.key, minval, self.key, maxval))
            else:
                return ('((properties ->> \'%s\')::time >= \'%s\'::time) AND '
                        '((properties ->> \'%s\')::time <= \'%s\'::time)' %
                        (self.key, minval, self.key, maxval))
        else:
            if minval is not None:
                return ('((properties ->> \'%s\')::time >= \'%s\'::time)' %
                        (self.key, minval))

            if maxval is not None:
                return ('((properties ->> \'%s\')::time <= \'%s\'::time)' %
                        (self.key, maxval))


class LookupField(Field):
    """
    A lookup value is a special kind of field the provides an pre-defined
    number of values as valid input values.
    """
    def validate_input(self, value):
        """
        Checks if the provided value matches the ID of one of the field's
        lookupvalues.

        Parameters
        ----------
        value : int
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(value)

        valid = False

        if value is not None:
            try:
                value = int(value)
                for lookupvalue in self.lookupvalues.all():
                    if lookupvalue.id == value:
                        valid = True
            except ValueError:
                pass
        else:
            valid = True

        if not valid:
            raise InputError('The value for lookup field %s is not an '
                             'accepted value for the field.' % self.name)

    def convert_from_string(self, value):
        """
        Returns the `value` of the field int.

        Parameters
        ----------
        value : str
            The value to be converted

        Return
        ------
        int
            Value of the field

        Notes
        -----
        Deprecated from version 0.6 on.
        """
        if value is None or len(value) == 0:
            return None

        return int(value)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Select box'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : List
            IDs of LookupValues that need to matched in order for the filter
            to apply.

        Return
        ------
        str
            SQL where-clause
        """
        return ('((properties ->> \'%s\')::int IN (%s))' %
                (self.key, ','.join(str(x) for x in rule)))


class LookupValue(models.Model):
    """
    Stores a single lookup value.
    """
    name = models.CharField(max_length=100)
    field = models.ForeignKey(LookupField, related_name='lookupvalues')
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )

    objects = LookupValueManager()

    class Meta:
        ordering = ['id']

    def delete(self):
        """
        Deletes the value by settings its status to `deleted`
        """
        self.status = STATUS.inactive
        self.save()


class MultipleLookupField(Field):
    def validate_input(self, provided_vals):
        """
        Checks if the provided value matches the ID of one of the field's
        lookupvalues.

        Parameters
        ----------
        value : int
            The value to be validated

        Notes
        -----
        Raises InputError if an invalid value is provided
        """
        self.validate_required(provided_vals)

        valid = True

        if provided_vals is not None:
            if isinstance(provided_vals, (str, unicode)):
                provided_vals = json.loads(provided_vals)

            accepted_values = [value.id for value in self.lookupvalues.all()]
            intersection = [
                val for val in provided_vals if val in accepted_values
            ]

            valid = len(intersection) == len(provided_vals)

        if not valid:
            raise InputError('One or more values for the multiple select '
                             'field %s is not an accepted value for the '
                             'field.' % self.name)

    def convert_from_string(self, value):
        """
        Returns the `value` of the field as List.

        Parameters
        ----------
        value : str
            The value to be converted

        Return
        ------
        List
            List of lookup value IDs

        Notes
        -----
        Deprecated from version 0.6 on.
        """
        if value is None or len(value) == 0:
            return None

        return json.loads(value)

    @property
    def type_name(self):
        """
        Returns a human readable name of the field.

        Return
        ------
        str
            The name of the field
        """
        return 'Multiple select'

    def get_filter(self, rule):
        """
        Returns the SQL where clause for the given field based on the rule.
        Used to filter data for a data grouping.

        Parameter
        ---------
        rule : List
            IDs of LookupValues that need to matched in order for the filter
            to apply.

        Return
        ------
        str
            SQL where-clause
        """
        return ('(regexp_split_to_array(btrim(properties ->> \'%s\', \'[]\'),'
                ' \',\')::int[] && ARRAY[%s])' % (self.key, ', '.join(rule)))


class MultipleLookupValue(models.Model):
    """
    Stores a multiple lookup value.
    """
    name = models.CharField(max_length=100)
    field = models.ForeignKey(MultipleLookupField, related_name='lookupvalues')
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )

    objects = LookupValueManager()

    class Meta:
        ordering = ['id']

    def delete(self):
        """
        Deletes the value by settings its status to `deleted`
        """
        self.status = STATUS.inactive
        self.save()



from django.conf import settings
from django.conf.urls.static import static

from django.conf.urls import patterns, include, url


urlpatterns = patterns(
    '',
    url(r'^', include('geokey.core.urls')),
    url(r'^', include('geokey_epicollect.urls', namespace='geokey_epicollect')),
    # url(r'^', include('geokey_cartodb.urls', namespace='geokey_cartodb')),
    url(r'^', include('geokey_communitymaps.urls', namespace='community_maps')),
    url(r'^', include('geokey_sapelli.urls', namespace='geokey_sapelli')),
    url(r'^', include('geokey_export.urls', namespace='geokey_export')),
    url(r'^', include('geokey_geotagx.urls', namespace='geokey_geotagx')),
) + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import json

from django.core.exceptions import ValidationError
from django.test import TestCase
from django.contrib.auth.models import AnonymousUser
from nose.tools import raises

from geokey.core.exceptions import MalformedRequestData
from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.categories.tests.model_factories import (
    CategoryFactory, TextFieldFactory, NumericFieldFactory
)

from ..serializers import (
    LocationContributionSerializer, ContributionSerializer, CommentSerializer
)

from ..models import Observation, Location
from .model_factories import (
    LocationFactory, ObservationFactory, CommentFactory
)


class LocationContributionSerializerTest(TestCase):
    def test_create_new_full_location(self):
        user = UserF.create()
        project = ProjectF.create()
        data = {
            "name": "Location",
            "description": "Location description",
            "geometry": '{"type": "Point","coordinates": [ '
                        '-0.144415497779846, 51.54671869005856]}',
            "private": True,
            "private_for_project": project.id
        }
        serializer = LocationContributionSerializer(
            data=data,
            context={'user': user}
        )
        if serializer.is_valid():
            serializer.save()

        self.assertEqual(Location.objects.count(), 1)
        self.assertEqual(Location.objects.all()[0].creator, user)

    def test_create_geometry_only(self):
        user = UserF.create()
        data = {
            "geometry": '{ "type": "Point","coordinates": [ '
                        '-0.144415497779846, 51.54671869005856] }',
        }
        serializer = LocationContributionSerializer(
            data=data,
            context={'user': user}
        )
        if serializer.is_valid():
            serializer.save()

        self.assertEqual(Location.objects.count(), 1)
        self.assertEqual(Location.objects.all()[0].creator, user)

    def test_update_full_location(self):
        user = UserF.create()
        location = LocationFactory.create()
        data = {
            "name": "Private Location",
            "private": True,
            "geometry": '{"type": "Point","coordinates": [ '
                        '-0.144415497779846, 51.54671869005856]}',
        }
        serializer = LocationContributionSerializer(
            location,
            data=data,
            context={'user': user}
        )
        if serializer.is_valid():
            serializer.save()

        ref = Location.objects.get(pk=location.id)
        self.assertEqual(ref.name, data.get('name'))
        self.assertEqual(
            json.loads(ref.geometry.geojson),
            json.loads(data.get('geometry'))
        )
        self.assertEqual(ref.private, data.get('private'))

    def test_update_geometry_only(self):
        user = UserF.create()
        location = LocationFactory.create()
        data = {
            "geometry": '{"type": "Point","coordinates": [ '
                        '-0.144415497779846, 51.54671869005856]}',
        }
        serializer = LocationContributionSerializer(
            location,
            data=data,
            context={'user': user}
        )
        if serializer.is_valid():
            serializer.save()

        ref = Location.objects.get(pk=location.id)
        self.assertEqual(ref.name, location.name)
        self.assertEqual(
            json.loads(ref.geometry.geojson),
            json.loads(data.get('geometry'))
        )


class ContributionSerializerTest(TestCase):
    def test_replace_null(self):
        properties = {
            'text': 'blah',
            'number': 2,
            'text_null': ''
        }

        serializer = ContributionSerializer(data={})

        result = serializer.replace_null(properties)
        self.assertIsNone(result.get('text_null'))
        self.assertEqual(result.get('number'), properties.get('number'))
        self.assertEqual(result.get('text'), properties.get('text'))

    def test_validate_location(self):
        project = ProjectF.create()
        serializer = ContributionSerializer(context={'user': project.creator})
        serializer._errors = {}
        serializer.validate_location(project, 8271839172)
        self.assertIsNotNone(serializer._errors.get('location'))

        project = ProjectF.create()
        location = LocationFactory.create(**{'private': True})
        serializer = ContributionSerializer(context={'user': project.creator})
        serializer._errors = {}
        serializer.validate_location(project, location.id)
        self.assertIsNotNone(serializer._errors.get('location'))

        project = ProjectF.create()
        location = LocationFactory.create()
        serializer = ContributionSerializer(context={'user': project.creator})
        serializer._errors = {}
        serializer.validate_location(project, location.id)
        self.assertEqual(serializer._errors, {})

    def test_get_display_field(self):
        observation = ObservationFactory(**{
            'display_field': 'text:blah'
        })

        serializer = ContributionSerializer(observation)
        display_field = serializer.get_display_field(observation)
        self.assertEqual(display_field.get('key'), 'text')
        self.assertEqual(display_field.get('value'), 'blah')


class ContributionSerializerIntegrationTests(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.category = CategoryFactory(**{
            'status': 'active',
            'project': self.project
        })

        TextFieldFactory.create(**{
            'key': 'key_1',
            'category': self.category,
            'order': 0
        })
        NumericFieldFactory.create(**{
            'key': 'key_2',
            'category': self.category,
            'order': 1
        })

    def test_create_observation(self):
        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True,
                "geometry": '{ "type": "Point", "coordinates": ['
                            '-0.134046077728271, 51.52439200896907] }',
            }
        }

        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid()
        serializer.save()

        result = serializer.data

        self.assertEqual(
            result.get('properties').get('key_1'),
            'value 1'
        )
        self.assertEqual(
            result.get('properties').get('key_2'),
            12
        )
        self.assertEqual(
            result.get('meta').get('category').get('id'),
            self.category.id)
        self.assertEqual(
            result.get('location').get('name'), 'UCL')
        self.assertEqual(
            result.get('location').get('description'),
            "UCL's main quad")
        self.assertEqual(
            json.loads(data['location']['geometry']),
            json.loads(result['location']['geometry'])
        )

    def test_create_without_speficfied_location(self):
        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "geometry": '{ "type": "Point", "coordinates": ['
                            '-0.134046077728271, 51.52439200896907] }',
            }
        }

        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid()
        serializer.save()
        result = serializer.data

        self.assertEqual(
            result.get('properties').get('key_1'),
            'value 1'
        )
        self.assertEqual(
            result.get('properties').get('key_2'),
            12
        )
        self.assertEqual(
            result.get('meta').get('category').get('id'),
            self.category.id)
        self.assertEqual(
            result.get('location').get('name'), None)
        self.assertEqual(
            result.get('location').get('description'),
            None)
        self.assertEqual(
            json.loads(data['location']['geometry']),
            json.loads(result['location']['geometry'])
        )

    def test_create_with_existing_location(self):
        location = LocationFactory.create()

        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "id": location.id,
                "geometry": location.geometry.geojson
            }
        }

        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid()
        serializer.save()
        result = serializer.data

        self.assertEqual(
            result.get('properties').get('key_1'),
            'value 1'
        )
        self.assertEqual(
            result.get('properties').get('key_2'),
            12
        )
        self.assertEqual(
            result.get('meta').get('category').get('id'),
            self.category.id)
        self.assertEqual(
            result.get('location').get('name'),
            location.name)
        self.assertEqual(
            result.get('location').get('description'),
            location.description)
        self.assertEqual(
            json.loads(data['location']['geometry']),
            json.loads(result['location']['geometry'])
        )

    @raises(ValidationError)
    def test_create_with_wrong_location(self):
        project = ProjectF()
        location = LocationFactory(**{
            'private': True,
            'private_for_project': project
        })

        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "id": location.id,
                "geometry": location.geometry.geojson
            }
        }
        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid(raise_exception=True)

    @raises(ValidationError)
    def test_create_with_inactive_category(self):
        self.category.status = 'inactive'
        self.category.save()

        data = {
            "location": {
                "geometry": '{ "type": "Point", "coordinates": ['
                            '-0.134046077728271, 51.52439200896907] }',
            },
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            }
        }
        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid(raise_exception=True)

    @raises(ValidationError)
    def test_create_with_private_location(self):
        location = LocationFactory(**{'private': True})
        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": 12
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "id": location.id,
                "geometry": location.geometry.geojson
            }
        }
        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid(raise_exception=True)

    @raises(ValidationError)
    def test_create_with_invalid_data(self):
        data = {
            "properties": {
                "key_1": "value 1",
                "key_2": "blah"
            },
            "meta": {
                "category": self.category.id
            },
            "location": {
                "name": "UCL",
                "description": "UCL's main quad",
                "private": True,
                "geometry": '{ "type": "Point", "coordinates": ['
                            '-0.134046077728271, 51.52439200896907] }',
            }
        }
        serializer = ContributionSerializer(
            data=data,
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid(raise_exception=True)

    def test_serialize_instance(self):
        observation = ObservationFactory.create(
            **{'properties': {'key': 'value'}}
        )
        TextFieldFactory.create(**{
            'key': 'key',
            'category': observation.category})
        CommentFactory.create(**{
            'commentto': observation
        })
        serializer = ContributionSerializer(
            observation,
            context={'user': self.contributor, 'project': self.project}
        )
        result = serializer.data

        self.assertEqual(
            result.get('properties').get('key'),
            'value'
        )
        self.assertEqual(
            result.get('meta').get('category').get('id'),
            observation.category.id)
        self.assertEqual(
            result.get('meta').get('num_comments'),
            observation.comments.count())
        self.assertEqual(
            result.get('meta').get('num_media'),
            observation.files_attached.count())
        self.assertEqual(
            result.get('location').get('name'),
            observation.location.name)
        self.assertEqual(
            result.get('location').get('description'),
            observation.location.description)
        self.assertEqual(
            json.loads(observation.location.geometry.geojson),
            json.loads(result['location']['geometry'])
        )

    def test_serialize_bulk(self):
        number = 20

        ObservationFactory.create_batch(number)
        observations = Observation.objects.prefetch_related(
            'location', 'category', 'creator', 'updator')

        serializer = ContributionSerializer(
            observations,
            many=True,
            context={'user': self.contributor, 'project': self.project}
        )
        result = serializer.data

        self.assertEqual(len(result), number)

        for f in result:
            self.assertIsNone(f.get('search_matches'))
            self.assertEqual(f.get('meta').get('num_comments'), 0)
            self.assertEqual(f.get('meta').get('num_media'), 0)

    def test_serialize_bulk_search(self):
        number = 20

        o_type = CategoryFactory.create()
        TextFieldFactory.create(**{
            'category': o_type,
            'key': 'field-1',
            'order': 0
        })
        TextFieldFactory.create(**{
            'category': o_type,
            'key': 'field-2',
            'order': 1
        })
        TextFieldFactory.create(**{
            'category': o_type,
            'key': 'field-3',
            'order': 2
        })

        ObservationFactory.create_batch(
            number,
            **{
                'properties': {
                    'field-1': 'blah',
                    'field-2': 'blabla',
                    'field-3': 'sddsdsfdsf'
                },
                'category': o_type
            }
        )
        observations = Observation.objects.prefetch_related(
            'location', 'category', 'creator', 'updator')

        serializer = ContributionSerializer(
            observations,
            many=True,
            context={
                'user': self.contributor,
                'project': self.project,
                'search': 'bla'
            }
        )
        result = serializer.data

        self.assertEqual(len(result), number)

        for f in result:
            self.assertIsNotNone(f.get('search_matches'))
            self.assertIsNone(
                f.get('search_matches').get('field-3')
            )

    def test_serialize_update(self):
        observation = ObservationFactory.create(
            **{'properties': {'number': 12}})
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': observation.category})
        serializer = ContributionSerializer(
            observation,
            data={
                'properties': {
                    'number': 15
                }
            },
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid()
        serializer.save()
        result = serializer.data

        self.assertEqual(
            result.get('properties').get('number'),
            15
        )

    @raises(ValidationError)
    def test_serialize_invalid_update(self):
        observation = ObservationFactory.create(
            **{'properties': {'number': 12}})
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': observation.category})
        serializer = ContributionSerializer(
            observation,
            data={
                'properties': {
                    'number': "blah"
                }
            },
            context={'user': self.contributor, 'project': self.project}
        )
        serializer.is_valid(raise_exception=True)

        o = Observation.objects.get(pk=observation.id)
        self.assertEqual(o.attributes.get('number'), 12)


class CommendSerializerTest(TestCase):
    def test_get_isowner(self):
        user = UserF.create()
        comment = CommentFactory.create(**{'creator': user})

        serializer = CommentSerializer(comment, context={'user': user})
        self.assertTrue(serializer.get_isowner(comment))

        serializer = CommentSerializer(
            comment, context={'user': UserF.create()})
        self.assertFalse(serializer.get_isowner(comment))

        serializer = CommentSerializer(
            comment, context={'user': AnonymousUser()})
        self.assertFalse(serializer.get_isowner(comment))



from django.conf.urls import patterns, url

from geokey.projects import views as project_views
from geokey.categories import views as category_views

from geokey.contributions.views import observations, comments, locations, media
from geokey.users.views import UserAPIView, ChangePasswordView


urlpatterns = patterns(
    '',
    # ###########################
    # USER
    # ###########################
    url(r'^user/$', UserAPIView.as_view(), name='user_api'),
    url(r'^user/password/change/$', ChangePasswordView.as_view(), name='changepassword'),

    # ###########################
    # PROJECTS
    # ###########################
    url(
        r'^projects/$',
        project_views.Projects.as_view(),
        name='project'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/$',
        project_views.SingleProject.as_view(),
        name='project_single'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/get-in-touch/$',
        project_views.ProjectContactAdmins.as_view(),
        name='project_contact_admins'),

    # ###########################
    # OBSERVATIONS
    # ###########################

    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/$',
        observations.ProjectObservations.as_view(),
        name='project_observations'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/search/$',
        observations.ContributionSearchAPIView.as_view(),
        name='contributions_search'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/(?P<observation_id>[0-9]+)/$',
        observations.SingleAllContributionAPIView.as_view(),
        name='project_single_observation'),

    # ###########################
    # LOCATIONS
    # ###########################

    url(
        r'^projects/(?P<project_id>[0-9]+)/locations/$',
        locations.Locations.as_view(),
        name='project_locations'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/locations/(?P<location_id>[0-9]+)/$',
        locations.SingleLocation.as_view(),
        name='project_single_location'),

    # ###########################
    # COMMENTS
    # ###########################
    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/(?P<observation_id>[0-9]+)/comments/$',
        comments.AllContributionsCommentsAPIView.as_view(),
        name='project_comments'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/(?P<observation_id>[0-9]+)/comments/(?P<comment_id>[0-9]+)/$',
        comments.AllContributionsSingleCommentAPIView.as_view(),
        name='project_single_comment'),

    # ###########################
    # MEDIA
    # ###########################

    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/(?P<contribution_id>[0-9]+)/media/$',
        media.AllContributionsMediaAPIView.as_view(),
        name='project_media'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/contributions/(?P<contribution_id>[0-9]+)/media/(?P<file_id>[0-9]+)/$',
        media.AllContributionsSingleMediaApiView.as_view(),
        name='project_single_media'),

    # ###########################
    # CATEGORIES
    # ###########################
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/$',
        category_views.SingleCategory.as_view(),
        name='category'),
)



from django.test import TestCase, TransactionTestCase
from nose.tools import raises

from ..models import User
from .model_factories import UserF


class TestCreateSuperUser(TestCase):
    def tearDown(self):
        User.objects.all().delete()

    def test(self):
        email = 'bla@example.com'
        display_name = 'superuser'
        password = 'password'
        user = User.objects.create_superuser(email, display_name, password)

        self.assertTrue(user.is_superuser)
        self.assertEqual(user.email, email)
        self.assertEqual(user.display_name, display_name)

    @raises(TypeError)
    def test_without_password(self):
        email = 'bla@example.com'
        display_name = 'superuser'
        User.objects.create_superuser(email, display_name)

        self.assertTrue(True)


class TestCreateUser(TransactionTestCase):
    def tearDown(self):
        User.objects.all().delete()

    def test(self):
        email = 'bla@example.com'
        display_name = 'user'
        password = 'password'
        user = User.objects.create_user(email, display_name, password)

        self.assertFalse(user.is_superuser)
        self.assertEqual(user.email, email)
        self.assertEqual(user.display_name, display_name)

    def test_without_password(self):
        email = 'bla@example.com'
        display_name = 'user'
        user = User.objects.create_user(email, display_name)

        self.assertFalse(user.is_superuser)
        self.assertEqual(user.email, email)
        self.assertEqual(user.display_name, display_name)


class TestGetUser(TestCase):
    def test(self):
        email = 'bla@example.com'
        display_name = 'user'
        password = 'password'
        user = User.objects.create_user(email, display_name, password)

        self.assertEqual(
            user, User.objects.get_by_natural_key(user.email)
        )
        self.assertEqual(
            user, User.objects.get_by_natural_key('BLA@example.com')
        )



import json

from django.test import TestCase
from django.core.urlresolvers import reverse

from rest_framework.test import APIRequestFactory, force_authenticate

from geokey.projects.tests.model_factories import UserF, ProjectF

from ..model_factories import LocationFactory
from geokey.contributions.views.locations import Locations, SingleLocation
from geokey.contributions.models import Location


class LocationApiTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.other_project = ProjectF.create()

        # Create 20 locations, 10 should be accessible for the project
        for x in range(0, 5):
            LocationFactory()
            LocationFactory(**{
                'private': True,
                'private_for_project': self.other_project
            })
            LocationFactory(**{
                'private': True,
                'private_for_project': self.project
            })
            LocationFactory(**{
                'private': True
            })

    def _get(self, user):
        url = reverse(
            'api:project_locations',
            kwargs={
                'project_id': self.project.id
            }
        )
        request = self.factory.get(url)
        force_authenticate(request, user=user)
        view = Locations.as_view()
        return view(request, project_id=self.project.id).render()

    def test_get_locations_with_admin(self):
        response = self._get(self.admin)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(
            len(json.loads(response.content).get('features')), 10
        )

    def test_get_locations_with_contributor(self):
        response = self._get(self.contributor)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(
            len(json.loads(response.content).get('features')), 10
        )

    def test_get_locations_with_non_member(self):
        response = self._get(self.non_member)
        self.assertEquals(response.status_code, 404)


class LocationQueryTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.project = ProjectF()
        LocationFactory.create(**{'name': 'Hyde Park'})
        LocationFactory.create(**{'description': 'hyde'})
        LocationFactory.create(**{'name': 'hyde park'})
        LocationFactory.create(**{'name': 'Regents Park'})

        self.url = reverse(
            'api:project_locations',
            kwargs={
                'project_id': self.project.id
            }
        )

    def test_hyd(self):
        request = self.factory.get(self.url + '?query=Hyd')
        force_authenticate(request, user=self.project.creator)
        view = Locations.as_view()
        response = view(request, project_id=self.project.id).render()

        response_json = json.loads(response.content)
        self.assertEqual(len(response_json.get('features')), 3)
        self.assertNotIn('Regents Park', response.content)

    def test_park(self):
        request = self.factory.get(self.url + '?query=park')
        force_authenticate(request, user=self.project.creator)
        view = Locations.as_view()
        response = view(request, project_id=self.project.id).render()

        response_json = json.loads(response.content)
        self.assertEqual(len(response_json.get('features')), 3)
        self.assertNotIn('"description": "hyde"', response.content)

    def test_regen(self):
        request = self.factory.get(self.url + '?query=regen')
        force_authenticate(request, user=self.project.creator)
        view = Locations.as_view()
        response = view(request, project_id=self.project.id).render()

        response_json = json.loads(response.content)
        self.assertEqual(len(response_json.get('features')), 1)
        self.assertNotIn('hyde', response.content)
        self.assertNotIn('Hyde Park', response.content)


class LocationUpdateApiTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.view_member = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

        self.location = LocationFactory(**{
            'private': True,
            'private_for_project': self.project
        })

    def patch(self, data, user, location_id=None):
        l_id = location_id or self.location.id
        url = reverse(
            'api:project_single_location',
            kwargs={
                'project_id': self.project.id,
                'location_id': l_id
            }
        )
        request = self.factory.patch(
            url, json.dumps(data), content_type='application/json')
        force_authenticate(request, user=user)
        view = SingleLocation.as_view()
        return view(
            request, project_id=self.project.id, location_id=l_id).render()

    def test_update_location_with_wrong_status(self):
        response = self.patch({'status': 'Bla'}, self.admin)
        self.assertEqual(response.status_code, 400)

        location = Location.objects.get(pk=self.location.id)
        self.assertEqual(location.status, self.location.status)

    def test_update_not_existing_location(self):
        response = self.patch(
            {'properties': {'name': 'UCL'}},
            self.admin,
            location_id=10000000000000000000000
        )
        self.assertEqual(response.status_code, 404)

    def test_update_private_location(self):
        private = LocationFactory(**{
            'private': True
        })
        response = self.patch(
            {'name': 'UCL'}, self.admin, location_id=private.id)
        self.assertEqual(response.status_code, 403)

        location = Location.objects.get(pk=private.id)
        self.assertEqual(location.name, private.name)

    def test_update_location_with_admin(self):
        response = self.patch(
            {
                'geometry': {
                    'type': 'Point',
                    'coordinates': [
                        -0.14061212539672852,
                        51.501763857255106
                    ]
                },
                'name': 'UCL'
            },
            self.admin
        )
        self.assertEqual(response.status_code, 200)
        location = Location.objects.get(pk=self.location.id)
        self.assertEqual(location.name, 'UCL')

    def test_update_location_with_contributor(self):
        response = self.patch({'description': 'main quad'}, self.contributor)
        self.assertEqual(response.status_code, 200)

        location = Location.objects.get(pk=self.location.id)
        self.assertEqual(location.description, 'main quad')

    def test_update_location_with_non_member(self):
        response = self.patch({'description': 'UCL'}, self.non_member)
        self.assertEqual(response.status_code, 404)

        location = Location.objects.get(pk=self.location.id)
        self.assertEqual(location.description, self.location.description)



import pytz
from datetime import datetime

from django.test import TestCase
from django.contrib.auth.models import AnonymousUser
from django.core import mail

from nose.tools import raises

from geokey.categories.tests.model_factories import (
    CategoryFactory, TextFieldFactory, NumericFieldFactory, LookupFieldFactory,
    LookupValueFactory, DateTimeFieldFactory, DateFieldFactory,
    TimeFieldFactory, MultipleLookupFieldFactory, MultipleLookupValueFactory
)
from geokey.contributions.tests.model_factories import ObservationFactory

from geokey.users.tests.model_factories import UserF, UserGroupF
from geokey.categories.models import Category

from .model_factories import ProjectF
from ..models import Project, Admins


class CreateProjectTest(TestCase):
    def test_create_project(self):
        creator = UserF.create()

        project = Project.create(
            'Test', 'Test desc', True, True, creator
        )
        self.assertIn(creator, project.admins.all())


class ProjectTest(TestCase):
    @raises(Project.DoesNotExist)
    def test_delete_project(self):
        project = ProjectF.create()
        project.delete()
        Project.objects.get(pk=project.id)

    def test_str(self):
        project = ProjectF.create(**{
            'name': 'Name',
            'status': 'inactive',
            'isprivate': False
        })
        self.assertEqual(
            str(project),
            'Name status: inactive private: False'
        )

    def test_get_role(self):
        admin = UserF.create()
        moderator = UserF.create()
        contributor = UserF.create()
        other = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_moderators=[moderator],
            add_contributors=[contributor]
        )

        self.assertEqual('administrator', project.get_role(admin))
        self.assertEqual('moderator', project.get_role(moderator))
        self.assertEqual('contributor', project.get_role(contributor))
        self.assertEqual('watcher', project.get_role(other))

    def test_re_order_categories(self):
        project = ProjectF.create()

        category_0 = CategoryFactory.create(**{'project': project})
        category_1 = CategoryFactory.create(**{'project': project})
        category_2 = CategoryFactory.create(**{'project': project})
        category_3 = CategoryFactory.create(**{'project': project})
        category_4 = CategoryFactory.create(**{'project': project})

        project.re_order_categories(
            [category_4.id, category_0.id, category_2.id,
             category_1.id,  category_3.id]
        )

        categories = project.categories.all()

        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0], category_4)
        self.assertEqual(categories[1], category_0)
        self.assertEqual(categories[2], category_2)
        self.assertEqual(categories[3], category_1)
        self.assertEqual(categories[4], category_3)

    def test_re_order_categories_with_false_category(self):
        project = ProjectF.create()

        category_0 = CategoryFactory.create(**{'project': project})
        category_1 = CategoryFactory.create(**{'project': project})
        category_2 = CategoryFactory.create(**{'project': project})
        category_3 = CategoryFactory.create(**{'project': project})
        category_4 = CategoryFactory.create(**{'project': project})

        try:
            project.re_order_categories(
                [category_4.id, category_0.id, category_2.id,
                 category_1.id,  5854]
            )
        except Category.DoesNotExist:
            categories = project.categories.all()

            self.assertTrue(categories.ordered)
            self.assertEqual(categories[0].order, 0)
            self.assertEqual(categories[1].order, 1)
            self.assertEqual(categories[2].order, 2)
            self.assertEqual(categories[3].order, 3)
            self.assertEqual(categories[4].order, 4)


class PrivateProjectTest(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': True,
            'everyone_contributes': 'false'
        })
        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertFalse(self.project.can_access(self.viewer_view))
        self.assertFalse(self.project.can_access(self.viewer))

        self.assertFalse(self.project.can_access(self.some_dude))
        self.assertFalse(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(
            self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class EveryoneContributesTest(TestCase):
    def test(self):
        project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'auth'
        })

        self.assertTrue(project.can_contribute(UserF.create()))
        self.assertFalse(project.can_contribute(AnonymousUser()))

        project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'true'
        })

        self.assertTrue(project.can_contribute(UserF.create()))
        self.assertTrue(project.can_contribute(AnonymousUser()))


class PublicProjectTest(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'false'
        })

        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertTrue(self.project.can_access(self.viewer_view))
        self.assertTrue(self.project.can_access(self.viewer))

        self.assertTrue(self.project.can_access(self.some_dude))
        self.assertTrue(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class ProjectContactAdminsTest(TestCase):
    def test_all_contacts(self):
        admin = UserF.create()
        contributor = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_contributors=[contributor]
        )

        project.contact_admins(email_user, 'Test email')
        # Should be 2 because project creator is admin too
        self.assertEquals(len(mail.outbox), 2)

    def test_selected_contacts(self):
        admin = UserF.create()
        contributor = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_contributors=[contributor]
        )

        admin_rel = Admins.objects.get(project=project, user=admin)
        admin_rel.contact = False
        admin_rel.save()

        project.contact_admins(email_user, 'Test email')
        self.assertEquals(len(mail.outbox), 1)


class ProjectGetDataTest(TestCase):
    def test_get_data_with_none_rule(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        category_2 = CategoryFactory(**{'project': project})
        category_3 = CategoryFactory(**{'project': project})

        UserGroupF.create(
            add_users=[user],
            **{'project': project, 'filters': None}
        )

        ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1}
        )
        ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_2}
        )
        ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_3}
        )

        self.assertEqual(project.get_all_contributions(user).count(), 15)

    def test_get_data_category_filter(self):
        user = UserF.create()
        project = ProjectF.create()

        category_1 = CategoryFactory(**{'project': project})
        category_2 = CategoryFactory(**{'project': project})

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {category_1.id: {}}
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'status': 'pending'}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_2}
            )
        self.assertEqual(project.get_all_contributions(user).count(), 10)

    def test_get_data_text_filter(self):
        user = UserF.create()
        project = ProjectF.create()

        category_1 = CategoryFactory(**{'project': project})
        TextFieldFactory(**{
            'key': 'text',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        TextFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {category_1.id: {'text': 'yes'}}
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'text': 'yes %s' % x}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'status': 'pending',
                'properties': {'text': 'yes %s' % x}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'text': 'no %s' % x}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': 'yes %s' % x}}
            )

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_number_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        NumericFieldFactory.create(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {category_1.id: {'number': {'minval': '15'}}}
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'number': 12}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'number': 20}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'number': 12}}
            )

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_max_number_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        NumericFieldFactory.create(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {category_1.id: {'number': {'maxval': '15'}}}
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'number': 12}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'number': 20}}
            )

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'number': 12}}
            )

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_max_number_filter(self):
            user = UserF.create()
            project = ProjectF.create()
            category_1 = CategoryFactory(**{'project': project})
            NumericFieldFactory.create(**{
                'key': 'number',
                'category': category_1
            })
            category_2 = CategoryFactory(**{'project': project})
            NumericFieldFactory.create(**{
                'key': 'bla',
                'category': category_2
            })

            UserGroupF.create(
                add_users=[user],
                **{
                    'project': project,
                    'filters': {
                        category_1.id: {'number': {
                            'minval': '10',
                            'maxval': '22'
                        }}
                    }
                }
            )

            for x in range(0, 5):
                ObservationFactory.create(**{
                    'project': project,
                    'category': category_1,
                    'properties': {'number': 5}}
                )

                ObservationFactory.create(**{
                    'project': project,
                    'category': category_1,
                    'properties': {'number': 12}}
                )

                ObservationFactory.create(**{
                    'project': project,
                    'category': category_1,
                    'properties': {'number': 20}}
                )

                ObservationFactory.create(**{
                    'project': project,
                    'category': category_1,
                    'properties': {'number': 25}}
                )

                ObservationFactory.create(**{
                    'project': project,
                    'category': category_2,
                    'properties': {'number': 12}}
                )

            self.assertEqual(project.get_all_contributions(user).count(), 10)

    def test_get_data_lookup_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})

        lookup_field = LookupFieldFactory(**{
            'key': 'lookup',
            'category': category_1
        })
        lookup_1 = LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        lookup_2 = LookupValueFactory(**{
            'name': 'Kermit',
            'field': lookup_field
        })
        category_2 = CategoryFactory(**{'project': project})
        lookup_field_2 = LookupFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })
        lookup_3 = LookupValueFactory(**{
            'name': 'Gonzo',
            'field': lookup_field_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'lookup': [lookup_1.id, lookup_2.id]}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'lookup': lookup_1.id}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'lookup': lookup_2.id}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': lookup_3.id}
            })
        self.assertEqual(project.get_all_contributions(user).count(), 10)

    def test_get_data_min_max_datetime_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        DateTimeFieldFactory(**{
            'key': 'date',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        DateTimeFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'date': {
                        'minval': '2014-01-01', 'maxval': '2014-06-09 00:00'}
                    }
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2014-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2013-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '2014-04-09'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_max_date_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'date',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'date': {
                        'minval': '2014-01-01', 'maxval': '2014-06-09'}
                    }
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2014-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2013-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '2014-04-09'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_date_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'date',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'date': {
                        'minval': '2014-01-01'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2014-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2013-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '2014-04-09'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_max_date_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'date',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        DateFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'date': {
                        'maxval': '2014-01-01'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2014-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'date': '2013-04-09'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '2014-04-09'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_max_time_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'time',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'time': {
                        'minval': '10:00', 'maxval': '12:00'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '11:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '18:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '11:00'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_max_inverse_time_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'time',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'time': {
                        'minval': '22:00', 'maxval': '8:00'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '2:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '18:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '2:00'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_min_time_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'time',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'time': {
                        'minval': '12:00'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '11:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '18:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '11:00'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_max_time_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'time',
            'category': category_1
        })
        category_2 = CategoryFactory(**{'project': project})
        TimeFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {'time': {
                        'maxval': '12:00'}}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '11:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'time': '18:00'}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': '11:00'}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_created_after(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {
                        'min_date': '2013-05-01 00:00:00'}
                }
            }
        )

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2014, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2013, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2012, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        self.assertEqual(project.get_all_contributions(user).count(), 10)

    def test_get_created_before(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {
                        'max_date': '2013-05-01 00:00:00'}
                }
            }
        )

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2014, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2013, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2012, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_created_before_and_after(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {
                        'min_date': '2013-01-01 00:00:00',
                        'max_date': '2013-10-01 00:00:00'}
                }
            }
        )

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2014, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2013, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        obs = ObservationFactory.create_batch(5, **{
            'project': project,
            'category': category_1
        })

        for o in obs:
            o.created_at = datetime(2012, 7, 23, 10, 34, 1, tzinfo=pytz.utc)
            o.save()

        self.assertEqual(project.get_all_contributions(user).count(), 5)

    def test_get_data_multiple_lookup_filter(self):
        user = UserF.create()
        project = ProjectF.create()
        category_1 = CategoryFactory(**{'project': project})
        lookup_field = MultipleLookupFieldFactory(**{
            'key': 'lookup',
            'category': category_1
        })
        lookup_1 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        lookup_2 = MultipleLookupValueFactory(**{
            'name': 'Kermit',
            'field': lookup_field
        })
        lookup_3 = MultipleLookupValueFactory(**{
            'name': 'Gonzo',
            'field': lookup_field
        })
        category_2 = CategoryFactory(**{'project': project})
        lookup_field_2 = MultipleLookupFieldFactory(**{
            'key': 'bla',
            'category': category_2
        })
        lookup_4 = MultipleLookupValueFactory(**{
            'name': 'Gonzo',
            'field': lookup_field_2
        })

        UserGroupF.create(
            add_users=[user],
            **{
                'project': project,
                'filters': {
                    category_1.id: {
                        'lookup': [str(lookup_1.id), str(lookup_2.id)]}
                }
            }
        )

        for x in range(0, 5):
            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'lookup': [lookup_1.id, lookup_3.id]}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_1,
                'properties': {'lookup': [lookup_2.id, lookup_3.id]}
            })

            ObservationFactory.create(**{
                'project': project,
                'category': category_2,
                'properties': {'bla': [lookup_4.id]}
            })

        self.assertEqual(project.get_all_contributions(user).count(), 10)



from django.test import TestCase
from django.core.exceptions import PermissionDenied

from nose.tools import raises

from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.projects.models import Project

from ..models import Field, Category, LookupValue

from .model_factories import (
    TextFieldFactory, CategoryFactory, LookupValueFactory
)


class CategoryManagerTest(TestCase):
    def test_access_with_projct_admin(self):
        admin = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': True}
        )

        CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })

        self.assertEqual(
            len(Category.objects.get_list(admin, project.id)), 2
        )

    def test_access_with_projct_contributor(self):
        contributor = UserF.create()

        project = ProjectF.create(
            add_contributors=[contributor],
            **{'isprivate': True}
        )

        active = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })

        types = Category.objects.get_list(contributor, project.id)
        self.assertEqual(len(types), 1)
        self.assertIn(active, types)

    @raises(Project.DoesNotExist)
    def test_access_with_projct_non_member(self):
        contributor = UserF.create()

        project = ProjectF.create()

        CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })
        Category.objects.get_list(contributor, project.id)

    def test_access_active_with_admin(self):
        admin = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': True}
        )

        active_type = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })

        self.assertEqual(active_type, Category.objects.get_single(
            admin, project.id, active_type.id))

    def test_access_inactive_with_admin(self):
        admin = UserF.create()
        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': True}
        )
        inactive_type = CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })
        self.assertEqual(inactive_type, Category.objects.get_single(
            admin, project.id, inactive_type.id))

    def test_access_active_with_contributor(self):
        contributor = UserF.create()

        project = ProjectF.create(
            add_contributors=[contributor],
            **{'isprivate': True}
        )

        active_type = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })

        self.assertEqual(active_type, Category.objects.get_single(
            contributor, project.id, active_type.id))

    @raises(PermissionDenied)
    def test_access_inactive_with_contributor(self):
        contributor = UserF.create()

        project = ProjectF.create(
            add_contributors=[contributor],
            **{'isprivate': True}
        )
        inactive_type = CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })
        Category.objects.get_single(
            contributor, project.id, inactive_type.id)

    @raises(Project.DoesNotExist)
    def test_access_active_with_non_member(self):
        contributor = UserF.create()

        project = ProjectF.create(**{
            'isprivate': True,
        })

        active_type = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })

        self.assertEqual(active_type, Category.objects.get_single(
            contributor, project.id, active_type.id))

    @raises(Project.DoesNotExist)
    def test_access_inactive_with_non_member(self):
        contributor = UserF.create()

        project = ProjectF.create(**{
            'isprivate': True,
        })
        inactive_type = CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })
        Category.objects.get_single(
            contributor, project.id, inactive_type.id)

    def test_admin_access_with_admin(self):
        admin = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': True}
        )

        active_type = CategoryFactory(**{
            'project': project
        })

        self.assertEqual(active_type, Category.objects.as_admin(
            admin, project.id, active_type.id))

    @raises(PermissionDenied)
    def test_admin_access_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )

        active_type = CategoryFactory(**{
            'project': project
        })

        Category.objects.as_admin(user, project.id, active_type.id)

    @raises(Project.DoesNotExist)
    def test_admin_access_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create(**{
            'isprivate': True
        })

        active_type = CategoryFactory(**{
            'project': project
        })

        Category.objects.as_admin(user, project.id, active_type.id)


class FieldManagerTest(TestCase):
    def test_access_fields_with_admin(self):
        admin = UserF.create()
        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        self.assertEqual(
            len(Field.objects.get_list(admin, project.id, category.id)),
            2
        )

    def test_access_active_field_with_admin(self):
        user = UserF.create()
        project = ProjectF.create(
            add_admins=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        self.assertEqual(
            field, Field.objects.get_single(
                user, project.id, category.id, field.id))

    def test_access_inactive_field_with_admin(self):
        user = UserF.create()
        project = ProjectF.create(
            add_admins=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        self.assertEqual(
            field, Field.objects.get_single(
                user, project.id, category.id, field.id))

    def test_admin_access_active_field_with_admin(self):
        user = UserF.create()
        project = ProjectF.create(
            add_admins=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        self.assertEqual(
            field, Field.objects.as_admin(
                user, project.id, category.id, field.id))

    def test_access_fields_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        inactive = TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        fields = Field.objects.get_list(user, project.id, category.id)
        self.assertEqual(len(fields), 1)
        self.assertNotIn(inactive, fields)

    def test_access_active_field_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        self.assertEqual(
            field, Field.objects.get_single(
                user, project.id, category.id, field.id))

    @raises(PermissionDenied)
    def test_access_active_field_inactive_cat_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'inactive'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        Field.objects.get_single(user, project.id, category.id, field.id)

    @raises(PermissionDenied)
    def test_access_inactive_field_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        Field.objects.get_single(
            user, project.id, category.id, field.id)

    @raises(PermissionDenied)
    def test_admin_access_active_field_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': True}
        )
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        Field.objects.as_admin(
            user, project.id, category.id, field.id)

    @raises(Project.DoesNotExist)
    def test_access_fields_with_non_member(self):
        user = UserF.create()
        project = ProjectF.create(**{'isprivate': True})
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        Field.objects.get_list(user, project.id, category.id)

    @raises(Project.DoesNotExist)
    def test_access_active_field_with_non_member(self):
        user = UserF.create()
        project = ProjectF.create(**{'isprivate': True})
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        Field.objects.get_single(
            user, project.id, category.id, field.id)

    @raises(Project.DoesNotExist)
    def test_access_inactive_field_with_non_member(self):
        user = UserF.create()
        project = ProjectF.create(**{'isprivate': True})
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'inactive',
            'category': category
        })
        Field.objects.get_single(
            user, project.id, category.id, field.id)

    @raises(Project.DoesNotExist)
    def test_admin_access_active_field_with_non_member(self):
        user = UserF.create()
        project = ProjectF.create(**{'isprivate': True})
        category = CategoryFactory(**{
            'project': project,
            'status': 'active'
        })
        field = TextFieldFactory.create(**{
            'status': 'active',
            'category': category
        })
        Field.objects.as_admin(
            user, project.id, category.id, field.id)


class LookupManagerTest(TestCase):
    def test(self):
        LookupValueFactory.create_batch(5, **{'status': 'active'})
        LookupValueFactory.create_batch(5, **{'status': 'deleted'})

        values = LookupValue.objects.active()
        self.assertEqual(len(values), 5)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0004_timefield'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'ordering': ['id']},
        ),
    ]
